@@ -1,10 +1,8 @@
-/* $Cambridge: exim/exim-src/src/rfc2047.c,v 1.4 2007/01/08 10:50:18 ph10 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2007 */
+/* Copyright (c) University of Cambridge 1995 - 2015 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This file contains a function for decoding message header lines that may
@@ -122,7 +120,7 @@ for (;; string = mimeword + 2)
   encoding = toupper((*q1ptr)[1]);
   **endptr = 0;
   if (encoding == 'B')
-    dlen = auth_b64decode(*q2ptr+1, dptrptr);
+    dlen = b64decode(*q2ptr+1, dptrptr);
   else if (encoding == 'Q')
     dlen = rfc2047_qpdecode(*q2ptr+1, dptrptr);
   **endptr = '?';   /* restore */
@@ -279,7 +277,7 @@ while (mimeword != NULL)
       else
         {
         DEBUG(D_any) debug_printf("iconv error translating \"%.*s\" to %s: "
-        "%s\n", endword + 2 - mimeword, mimeword, target, strerror(errno));
+        "%s\n", (int)(endword + 2 - mimeword), mimeword, target, strerror(errno));
         }
       }
 
